
Imports System
Imports System.Runtime.Serialization
Imports System.Collections
Imports System.Diagnostics

Imports Syncfusion.Windows.Forms.Diagram


'/ <summary>
'/ Each employee in the organization will be represented in the diagram as a Symbol node
'/ </summary>
<Serializable()>  _
Public Class EmployeeSymbol
    Inherits Symbol
    Private rect As RoundRect = Nothing
    Private nameLbl As Label = Nothing
    Private titleLbl As Label = Nothing
    Private strManagerID As String = String.Empty
    Private nRecSubEmployeeCount As Integer = 0
    
    
    Public Sub New()
        Me.rect = New RoundRect(0, 0, 60, 25)
        Me.LineStyle.LineWidth = 1
        Me.rect.FillStyle.Type = Syncfusion.Windows.Forms.Diagram.FillStyle.FillType.LinearGradient
        Me.rect.FillStyle.GradientStartColor = System.Drawing.Color.Orange
        Me.rect.FillStyle.Color = System.Drawing.Color.OrangeRed
        Me.AppendChild(Me.rect)
        Me.nameLbl = Me.AddLabel("", BoxPosition.Center)
        Me.nameLbl.OffsetY = - 5
        Me.nameLbl.FontStyle.Name = "Verdana"
        Me.nameLbl.FontStyle.Size = 8
        Me.nameLbl.FontStyle.Style = System.Drawing.FontStyle.Regular
        Me.nameLbl.BackgroundStyle.Color = System.Drawing.Color.Transparent
        Me.nameLbl.ReadOnly = True
        Me.nameLbl.FitBlackBox = True
        Me.nameLbl.WrapText = False
        Me.nameLbl.NoClip = False
        Me.titleLbl = Me.AddLabel("", BoxPosition.Center)
        Me.titleLbl.FontStyle.Name = "Verdana"
        Me.titleLbl.FontStyle.Size = 7
        Me.titleLbl.FontStyle.Style = System.Drawing.FontStyle.Italic
        Me.titleLbl.OffsetY = 7
        Me.titleLbl.BackgroundStyle.Color = System.Drawing.Color.Transparent
        Me.titleLbl.FitBlackBox = True
        Me.titleLbl.WrapText = False
        Me.titleLbl.NoClip = False
        Me.titleLbl.ReadOnly = True
    End Sub 'New
    
    
    Public Sub New(src As EmployeeSymbol)
        MyBase.New(src)
    End Sub 'New
    
    
    ' Serialization constructor
    Protected Sub New(info As SerializationInfo, context As StreamingContext)
        MyBase.New(info, context)
        Me.strManagerID = info.GetString("ManagerID")
        Me.nRecSubEmployeeCount = info.GetInt32("RecSubEmployeeCount")
    End Sub 'New
    
    
    Protected Overrides Sub GetObjectData(info As SerializationInfo, context As StreamingContext)
        MyBase.GetObjectData(info, context)
        
        info.AddValue("ManagerID", Me.strManagerID)
        info.AddValue("RecSubEmployeeCount", Me.nRecSubEmployeeCount)
    End Sub 'GetObjectData
    
    
    Public Property EmployeeName() As String
        Get
            Return Me.nameLbl.Text
        End Get
        Set
            Me.nameLbl.Text = value
            Me.nameLbl.Size = Me.nameLbl.SizeToText(Me.Size)
        End Set
    End Property
    
    
    Public Property EmployeeID() As String
        Get
            Return Me.titleLbl.Text
        End Get
        Set
            Me.titleLbl.Text = value
            Me.titleLbl.Size = Me.titleLbl.SizeToText(Me.Size)
        End Set
    End Property
    
    
    Public Property ManagerID() As String
        Get
            Return Me.strManagerID
        End Get
        Set
            If Me.strManagerID <> value Then
                Me.strManagerID = value
            End If
        End Set
    End Property
     
    Public Overrides Function AcceptConnection(sourcePort As Port, targetPort As Port) As Boolean
        Dim accept As Boolean = False
        
        ' First, determine which port belongs to the foreign port container.
        Dim foreignPort As Port = Nothing
        If Me.Ports.Contains(sourcePort) OrElse Me.CenterPort Is sourcePort Then
            foreignPort = targetPort
        ElseIf Me.Ports.Contains(targetPort) OrElse Me.CenterPort Is targetPort Then
            foreignPort = sourcePort
        End If

        If foreignPort Is Nothing Then
            Return False ' This should never happen
        End If

        ' Now get the foreign port container and check to see if it is
        ' a Link object. If so, then check to see if the foreign port
        ' is the link's tail port or head port. If it's the tail port,
        ' then always allow the connection, because a member symbol can
        ' have any number of edges leaving it. If it's the head port,
        ' then only allow the connection if the member symbol has no
        ' other edges entering it yet.
        Dim foreignContainer As IPortContainer = foreignPort.Container
        If Not (foreignContainer Is Nothing) Then
            Dim newlink As Link = foreignContainer
          
            If Not (newlink Is Nothing) Then
                If foreignPort Is newlink.TailPort Then
                    accept = True
                Else
                    Dim curNode As IGraphNode = Me
                    If Not (curNode Is Nothing) Then
                        Dim edgesEntering As ICollection = curNode.EdgesEntering
                        If edgesEntering.Count = 0 Then
                            accept = True
                        End If
                    End If
                End If
            End If
        End If

        Return accept
    End Function 'AcceptConnection
    
    
    Public ReadOnly Property Rank() As Integer
        Get
            Dim rankvalue As Integer = 1
            Dim thisNode As IGraphNode = Me
            Dim curNode As IGraphNode = thisNode
            Dim cyclicalPathFound As Boolean = False
            
            While Not (curNode Is Nothing) AndAlso Not cyclicalPathFound
                Dim edgesEntering As ICollection = curNode.EdgesEntering
                If edgesEntering.Count > 0 Then
                    Dim enumEdgesEntering As IEnumerator = edgesEntering.GetEnumerator()
                    If Not (enumEdgesEntering Is Nothing) Then
                        enumEdgesEntering.MoveNext()
                        Dim incomingEdge As IGraphEdge = enumEdgesEntering.Current
                        If Not (incomingEdge Is Nothing) Then
                            curNode = incomingEdge.FromNode

                            If curNode Is thisNode Then
                                cyclicalPathFound = True
                            Else
                                rankvalue += 1
                            End If
                        End If
                    End If
                Else
                    curNode = Nothing
                End If
            End While

            Return rankvalue
        End Get
    End Property
    
    
    Public ReadOnly Property Manager() As EmployeeSymbol
        Get
            Dim mgr As EmployeeSymbol = Nothing
            Dim thisNode As IGraphNode = Me
            Dim fromNode As IGraphNode
            
            If Not (thisNode Is Nothing) Then
                Dim edgesEntering As ICollection = thisNode.EdgesEntering
                If edgesEntering.Count > 0 Then
                    Dim enumEdgesEntering As IEnumerator = edgesEntering.GetEnumerator()
                    If Not (enumEdgesEntering Is Nothing) Then
                        enumEdgesEntering.MoveNext()
                        Dim incomingEdge As IGraphEdge = enumEdgesEntering.Current
                        If Not (incomingEdge Is Nothing) Then
                            fromNode = incomingEdge.FromNode
                            If Not (fromNode Is Nothing) Then
                                mgr = fromNode
                            End If
                        End If
                    End If
                End If
            End If
            
            Return mgr
        End Get
    End Property
    
    
    Public ReadOnly Property Employees() As ICollection
        Get
            Dim emplylist As New ArrayList
            Dim thisNode As IGraphNode = Me

            If Not (thisNode Is Nothing) Then
                Dim edgesLeaving As ICollection = thisNode.EdgesLeaving
                If Not (edgesLeaving Is Nothing) Then
                    Dim curEdge As IGraphEdge
                    For Each curEdge In  edgesLeaving
                        Dim curEmployee As EmployeeSymbol = curEdge.ToNode
                        If Not (curEmployee Is Nothing) Then
                            emplylist.Add(curEmployee)
                        End If
                    Next curEdge
                End If
            End If
            
            Return emplylist
        End Get
    End Property
    
    
    Public Property RecSubEmployeeCount() As Integer
        Get
            Return Me.nRecSubEmployeeCount
        End Get
        Set
            Me.nRecSubEmployeeCount = value
        End Set
    End Property
End Class 'EmployeeSymbol