
Imports System
Imports System.Collections
Imports System.ComponentModel

Imports Syncfusion.Windows.Forms.Diagram


'/ <summary>
'/ The OrgChartManager class is an implementor of the Essential Diagram LayoutManager functionality.
'/ This is a very simple layout management algorithm that calculates the sub-employees associated with 
'/ each employee and allocates space accordingly. More efficient management strategies may be used for 
'/ better diagram layout.
'/ </summary>

Public Class OrgChartManager
    Inherits LayoutManager
    
    Public Sub New()
    End Sub 'New
    
    
    Public Sub New(mdl As Model)
        MyBase.New(mdl)
    End Sub 'New
    
    
    Public Overrides Function UpdateLayout(contextInfo As Object) As Boolean
        Dim changesMade As Boolean = True
        
        If Not (Me.Model Is Nothing) Then
            Dim topRanking As New ArrayList()
            
            Dim curNode As INode
            For Each curNode In Me.Model.Nodes
                If TypeOf curNode Is EmployeeSymbol Then
                    Dim memberSym As EmployeeSymbol = curNode
                    If Not (memberSym Is Nothing) Then
                        If memberSym.Rank = 1 Then
                            topRanking.Add(memberSym)
                        End If
                    End If
                End If
            Next curNode

            Dim numTopRanking As Integer = topRanking.Count

            If numTopRanking > 0 Then
                Dim range As Single = Me.Model.Width / System.Convert.ToSingle(numTopRanking)
                Dim curLeft As Single = Me.Model.Bounds.Left
                Dim curRight As Single = curLeft
                Dim curMember As EmployeeSymbol
                For Each curMember In topRanking
                    ' Assuming a 4 pixel bufer between adjacent symbols, the space allocated for each of the 
                    ' top ranking symbols is calculated based on a recursive count of the subemployees associated 
                    ' with that top level employee.
                    Dim numsubemployees As Integer = curMember.RecSubEmployeeCount
                    If numsubemployees <= 0 Then
                        numsubemployees = 1
                    End If
                    curRight += (curMember.Width + 4) * numsubemployees
                    Me.PositionMember(curMember, curLeft, curRight)
                    curLeft = curRight
                Next curMember
            End If
        End If
        
        If changesMade Then
            Me.OnLayoutUpdated(New System.EventArgs())
        End If
        
        Return changesMade
    End Function 'UpdateLayout
    
    
    Private Sub PositionMember(memberSym As EmployeeSymbol, left As Single, right As Single)
        Dim rank As Integer = memberSym.Rank
        If rank = 1 Then
            memberSym.Y = verticalStartOffset
        Else
            memberSym.Y = verticalStartOffset + (rank - 1) * Me.vSpacing
        End If
        Dim symWidth As Single = memberSym.Width
        Dim rangeCenter As Single = (left + right) / 2.0F
        memberSym.X = rangeCenter - symWidth / 2.0F

        Dim employees As ICollection = memberSym.Employees
        Dim numEmployees As Integer = memberSym.RecSubEmployeeCount
        If numEmployees > 0 Then
            Dim curLeft As Single = left
            Dim curRight As Single = curLeft
            Dim employee As EmployeeSymbol
            For Each employee In employees
                Dim numsubemployees As Integer = employee.RecSubEmployeeCount
                If numsubemployees <= 0 Then
                    numsubemployees = 1
                End If
                curRight += (employee.Width + 4) * numsubemployees
                Me.PositionMember(employee, curLeft, curRight)
                curLeft = curRight
            Next employee
        End If
    End Sub 'PositionMember


    Public Property HorizontalSpacing() As Single
        Get
            Return Me.hSpacing
        End Get
        Set(ByVal Value As Single)
            Me.hSpacing = Value
        End Set
    End Property

    Public Property VerticalSpacing() As Single
        Get
            Return Me.vSpacing
        End Get
        Set(ByVal Value As Single)
            Me.vSpacing = Value
        End Set
    End Property
    Private verticalStartOffset As Single = 50.0F
    Private hSpacing As Single = 100.0F
    Private vSpacing As Single = 140.0F
End Class 'OrgChartManager